#---------------------------------------------------------------------------
#
#   SuPy GUI - Widget - Generic
#
#---------------------------------------------------------------------------

from properties import Properties, overridable_property

class GWidget(Properties):

	x = overridable_property('x', "Horizontal position in container")
	y = overridable_property('y', "Vertical position in container")
	pos = overridable_property('pos', "Position in container")
	width = overridable_property('width', "Width of widget")
	height = overridable_property('height', "Height of widget")
	size = overridable_property('size', "Size of widget")
	left = overridable_property('left', "Left edge of bounds")
	top = overridable_property('top', "Top edge of bounds")
	right = overridable_property('right', "Right edge of bounds")
	bottom = overridable_property('bottom', "Bottom edge of bounds")
	bounds = overridable_property('bounds', "Bounding rectangle in container coords")
	container = overridable_property('container', "Containing widget")
	anchor = overridable_property('anchor', "Layout behaviour when container resized")
	
	_container = None
	
	def destroy(self):
		self.container = None
	
	def get_x(self):
		return self.bounds[0]
	
	def set_x(self, x):
		b = self.bounds
		self.bounds = (x, b[1], b[2], b[3])

	def get_y(self):
		return self.bounds[1]
	
	def set_y(self, y):
		b = self.bounds
		self.bounds = (b[0], y, b[2], b[3])

	def get_pos(self):
		return self.bounds[0:1]
	
	def set_pos(self, p):
		x, y = p
		l, t, r, b = self.bounds
		self.bounds = (x, y, x + r - l, y + b - t)
	
	def get_width(self):
		b = self.bounds
		return b[2] - b[0]
	
	def set_width(self, w):
		l, t, r, b = self.bounds
		self.bounds = l, t, l + w, b
	
	def get_height(self):
		b = self.bounds
		return b[3] - b[1]
	
	def set_height(self, h):
		l, t, r, b = self.bounds
		self.bounds = l, t, r, t + h
	
	def get_size(self):
		l, t, r, b = self.bounds
		return r - l, b - t
	
	def set_size(self, size):
		w, h = size
		l, t, r, b = self.bounds
		self.bounds = l, t, l + w, t + h

	def get_left(self):
		return self.bounds[0]

	def set_left(self, x):
		b = self.bounds
		self.bounds = (x, b[1], b[2], b[3])

	def get_top(self):
		return self.bounds[1]
	
	def set_top(self, y):
		b = self.bounds
		self.bounds = (b[0], y, b[2], b[3])

	def get_right(self):
		return self.bounds[0]

	def set_right(self, x):
		b = self.bounds
		self.bounds = (b[0], b[1], x, b[3])

	def get_bottom(self):
		return self.bounds[1]
	
	def set_bottom(self, y):
		b = self.bounds
		self.bounds = (b[0], b[1], b[2], y)

	def get_container(self):
		return self._container

	def set_container(self, new):
		old = self._container
		if old is not new:
			if old:
				old._remove(self)
				old._contents.remove(self)
			if new:
				new._add(self)
				new._contents.append(self)
			self._container = new

	def handle(self, message, *args):
		method = getattr(self, message, None)
		if method:
			return method(*args)
		else:
			outer = self.next_handler()
			if outer:
				return outer.handle(message, *args)

	def next_handler(self):
		return self.container
